% MAIN2_AGPvsKR_trainingTime


% BENCHMARK 2
% 
% advanced gaussian process (AGP) vs kriging (KR)
% 
% This code compares the effciency of these 2 ML models by measuring their
% overall training time with multiple dataset sizes, while using a single 
% processor thread. The pattern is generated the BY modified 3D Ackley 
% function over a 3D clustered dataset distribution.



clear all
close all
clc

pattern = benchmark3DPatternExample(5000);
s = pattern.input;
t = pattern.output;


modelParameters = ' ';

sampleSize = 1500:500:5000;
nSamples=size(sampleSize,2);
timeMeasured = zeros(nSamples,2);

parMdl.fitrgpPar = " ,'Basis','linear','FitMethod','exact','PredictMethod','exact' ";

for i=1:nSamples
    
    s1=[];t1=[];
    s1=s(1:sampleSize(i),:);
    t1=t(1:sampleSize(i),:);

    tic
    modelTPS = trainAGP(s1,t1,parMdl);
    timeMeasured(i,1)=toc;
    fprintf('AGP sample size = %6d; training time = %15s \n',sampleSize(i),timeString(toc));
    
    tic
    gprMdl = fitrgp(s1,t1,'Basis','linear',...
        'FitMethod','exact','PredictMethod','exact');
    timeMeasured(i,2)=toc;
    fprintf('KR  sample size = %6d; training time = %15s  \n',sampleSize(i),timeString(toc));
end
save('temp1.mat')

% plot
hold on
plot(sampleSize(1:end),timeMeasured(1:end,1),'b-o','lineWidth',2)
plot(sampleSize(1:end),timeMeasured(1:end,2),'r-o','lineWidth',2)
grid on
% grid minor
xlabel('Sample Size');
ylabel('Training Time (sec)');
legend('AGP','KR')


% print results
fprintf('\n\n\n-------------------------------------------------------------\n');
fprintf('-                      training time                        -\n');
fprintf('-------------------------------------------------------------\n');
fprintf('dataset size | %16s | %12s | %10s |\n','KR','AGP','ratio');
fprintf('-------------------------------------------------------------\n');
for i=1:size(sampleSize,2)
    fprintf('%-11d  | %16s | %12s | %10.2f |\n',sampleSize(i),timeString(timeMeasured(i,2)),timeString(timeMeasured(i,1)),(timeMeasured(i,2)/timeMeasured(i,1)));
end
fprintf('-------------------------------------------------------------\n\n\n');
