% MAIN_callBenchmarkAGP


% BENCHMARK 
% 
% advanced gaussian process (AGP) vs kriging (KR)
% 
% This code compares the effciency of these 2 ML models by measuring their
% overall training time and the average error of the forecasted output of
% each model, while using a single processor thread. The pattern is 
% generated by the modified 3D Ackley function over a clustered dataset 
% distribution.


clear all
clc

% to be adjusted by the user
% define pattern
% patternSize = 1.5e3;
patternSize = 5e3;
% patternSize = 1e4;


% generate pattern
pattern = benchmark3DPatternExample(patternSize);
s = pattern.input;
t = pattern.output;


sizePattern = size(t,1);

s = pattern.input;
t = pattern.output;




% ---------------------------
% ---------------------------
% -----
% ----- AGP model

% Configure AGP parameters like KR parameters in fitrgp MATLAB function:
parMdl.fitrgpPar = " ,'Basis','linear','FitMethod','exact','PredictMethod','exact' ";
% parMdl.fitrgpPar = "  ";

fprintf('-------------------\n');
fprintf('----- AGP model\n\n');

% train AGP model
fprintf('training AGP model...\n');
modelAGP = trainAGP(s,t,parMdl);

% model specs
fprintf('AGP model with Kriging precision with %5d 3D inputs is trained.\n', modelAGP.nInputs);
fprintf('The total training time is %10s\n\n', timeString(modelAGP.trainingTimeSec));


% test precision of the AGP model by forecasting all dataset inputs
forecastedOutputAGP = forecastAGP(s,modelAGP); 

% Note that the inference variable s can be a vector or a single input.
% In this case s is all the dataset, to be forecasted using the AGP model.

% calc normalized output error
delta_output = max(t) - min(t);
norm_t = (t-min(t))./delta_output;
norm_y = (forecastedOutputAGP-min(t))./delta_output;
error1 = abs(norm_t-norm_y);
forecastedPercentualErrorAGP = sum(error1)*(100/sizePattern);

fprintf('average percentual error of AGP model \nfor the forecasted output is %12.5f %% \n',forecastedPercentualErrorAGP);
fprintf('\n----- AGP model\n');
fprintf('-------------------\n\n\n\n');



% ----- AGP model
% -----
% ---------------------------
% ---------------------------




% ---------------------------
% ---------------------------
% -----
% ----- KR matlab model


fprintf('-------------------\n');
fprintf('----- KR model - matlab function\n\n');


% train KR matlab model
fprintf('training KR model...\n');
tic
gprMdl = fitrgp(s,t,'Basis','linear','FitMethod','exact','PredictMethod','exact');
KRtrainingTime =toc;

fprintf('Kriging model with %5d 3D inputs is trained.\n', modelAGP.nInputs);

% test precision of the AGP model by forecasting all dataset inputs
forecastedOutputKR = predict(gprMdl,s);
fprintf('The total training time is %s\n\n', timeString(KRtrainingTime));

% Note that the inference variable s can be a vector or a single input.
% In this case s is all the dataset, to be forecasted using the AGP model.

% calc normalized output error
delta_output = max(t) - min(t);
norm_t = (t-min(t))./delta_output;
norm_y = (forecastedOutputKR-min(t))./delta_output;
error1 = abs(norm_t-norm_y);
forecastedPercentualErrorKR = sum(error1)*(100/sizePattern);

fprintf('average percentual error of KR model \nfor the forecasted output is %12.5f %% \n',forecastedPercentualErrorKR);

fprintf('\n----- KR model\n');
fprintf('-------------------\n\n');

% ----- KR matlab model
% -----
% ---------------------------
% ---------------------------




